/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:50 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for DISTR_DISCR
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_distr_discr_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */

#define COMPARE_SAMPLE_SIZE  (500)
#define VIOLATE_SAMPLE_SIZE   (20)

double pmf(int k, const UNUR_DISTR *distr);
double icdf(int k, const UNUR_DISTR *distr);





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_distr_discr_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_distr_discr_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(898100);
	unur_urng_fish_seed(898100);
	unur_urng_mstd_seed(898100);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(898100)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {898100,898100+1,898100+2,898100+3,898100+4,898100+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("distr_discr: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 34, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 34, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 37, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 37, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 40, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 40, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 43, (unur_distr_discr_set_pmfstr( distr, "pmf" )) );
n_tests_failed += check_errorcode( TESTLOG, 43, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 46, (unur_distr_discr_set_cdfstr( distr, "cdf" )) );
n_tests_failed += check_errorcode( TESTLOG, 46, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 49, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 49, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 52, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 52, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 55, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 55, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 58, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 58, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 65, (unur_distr_discr_set_pv( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 65, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 68, (unur_distr_discr_set_pmf( distr, pmf )) );
n_tests_failed += check_errorcode( TESTLOG, 68, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 71, (unur_distr_discr_set_cdf( distr, icdf )) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 74, (unur_distr_discr_set_pmfstr( distr, "exp(-k)" )) );
n_tests_failed += check_errorcode( TESTLOG, 74, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 77, (unur_distr_discr_set_cdfstr( distr, "exp(-k)" )) );
n_tests_failed += check_errorcode( TESTLOG, 77, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 80, (unur_distr_discr_set_pmfparams( distr, NULL, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 80, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 83, (unur_distr_discr_set_domain( distr, 0, 10 )) );
n_tests_failed += check_errorcode( TESTLOG, 83, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 86, (unur_distr_discr_set_mode( distr, 0 )) );
n_tests_failed += check_errorcode( TESTLOG, 86, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 89, (unur_distr_discr_set_pmfsum( distr, 1. )) );
n_tests_failed += check_errorcode( TESTLOG, 89, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 96, (unur_distr_discr_set_pmf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 96, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 99, (unur_distr_discr_set_cdf( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 99, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 102, (unur_distr_discr_set_pmfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 102, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 105, (unur_distr_discr_set_cdfstr( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 105, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 108, (unur_distr_discr_set_pmfparams( distr, NULL, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 108, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid parameters */
{
UNUR_DISTR *distr = NULL;
   double pv[] = {1.,2.,3.};
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 116, (unur_distr_discr_set_pmfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 116, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 119, (unur_distr_discr_set_pmfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 119, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 122, (unur_distr_discr_set_cdfstr( distr, "" )) );
n_tests_failed += check_errorcode( TESTLOG, 122, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 125, (unur_distr_discr_set_cdfstr( distr, "beta(2.)" )) );
n_tests_failed += check_errorcode( TESTLOG, 125, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 128, (unur_distr_discr_set_pmfparams( distr, pv, -1 )) );
n_tests_failed += check_errorcode( TESTLOG, 128, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 131, (unur_distr_discr_set_pmfparams( distr, pv, UNUR_DISTR_MAXPARAMS +1 )) );
n_tests_failed += check_errorcode( TESTLOG, 131, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 134, (unur_distr_discr_set_domain( distr, 10, 1 )) );
n_tests_failed += check_errorcode( TESTLOG, 134, UNUR_ERR_DISTR_SET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 137, (unur_distr_discr_set_pmfsum( distr, -1. )) );
n_tests_failed += check_errorcode( TESTLOG, 137, UNUR_ERR_DISTR_SET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 148, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 148, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 151, ((void *) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 151, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 154, ((void *) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 154, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 157, (unur_distr_discr_get_pmfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 157, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 160, (unur_distr_discr_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 160, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 163, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 163, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 166, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 166, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 169, (unur_distr_discr_make_pv( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 169, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   int ia, ib;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 177, ((void *) unur_distr_discr_get_pmf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 177, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 180, ((void *) unur_distr_discr_get_cdf( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 180, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 183, (unur_distr_discr_get_pmfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 183, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 186, (unur_distr_discr_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 186, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 189, (unur_distr_discr_get_pv( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 189, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 192, (unur_distr_discr_get_domain( distr, &ia, &ib )) );
n_tests_failed += check_errorcode( TESTLOG, 192, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 195, (unur_distr_discr_get_pmfparams( distr, NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 195, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 198, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 198, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 201, (unur_distr_discr_make_pv( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 201, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* parameters not unknown */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 208, (unur_distr_discr_get_pmfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 208, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 211, (unur_distr_discr_get_cdfstr( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 211, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 214, (unur_distr_discr_get_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 214, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_INTMAX( TESTLOG, 217, (unur_distr_discr_get_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 217, UNUR_ERR_DISTR_GET );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 220, (unur_distr_discr_make_pv( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 220, UNUR_ERR_DISTR_GET );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 231, (unur_distr_discr_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 231, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 234, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 234, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution type */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 241, (unur_distr_discr_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 241, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 244, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 244, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 251, (unur_distr_discr_upd_mode( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 251, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_setfailed( TESTLOG, 254, (unur_distr_discr_upd_pmfsum( distr )) );
n_tests_failed += check_errorcode( TESTLOG, 254, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_DISTR *distr = NULL;
   distr = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 273, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 273, UNUR_ERR_NULL );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 276, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 276, UNUR_ERR_NULL );
unur_distr_free(distr);
}

/* invalid distribution object */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_cont_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 284, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 284, UNUR_ERR_DISTR_INVALID );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 287, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 287, UNUR_ERR_DISTR_INVALID );
unur_distr_free(distr);
}

/* missing data */
{
UNUR_DISTR *distr = NULL;
   distr = unur_distr_discr_new(); 

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 295, (unur_distr_discr_eval_pmf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 295, UNUR_ERR_DISTR_DATA );

unur_errno = 0;
n_tests_failed += check_expected_INFINITY( TESTLOG, 298, (unur_distr_discr_eval_cdf( 1., distr )) );
n_tests_failed += check_errorcode( TESTLOG, 298, UNUR_ERR_DISTR_DATA );
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/


double pmf(int k, const UNUR_DISTR *distr) { return 1.; }
double icdf(int k, const UNUR_DISTR *distr) { return 1.; }



/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_distr_discr_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_distr_discr_set_verify(UNUR_PAR *par, int verify) {return 0;}
