/*
	file automatically generated by make_test_files.pl
	Wed Mar 12 18:04:52 2003
*/

/*****************************************************************************
 *                                                                           *
 *          UNURAN -- Universal Non-Uniform Random number generator          *
 *                                                                           *
 *****************************************************************************/
    
/**
 ** Tests for STRINGPARSER
 **/
    
/*---------------------------------------------------------------------------*/
#include "testunuran.h"
/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* global variables                                                          */

static FILE *TESTLOG;               /* test log file                         */
static FILE *UNURANLOG;             /* unuran log file                       */

static int test_ok = TRUE;          /* all tests ok (boolean)                */
static int n_tests_failed;          /* number of failed tests                */

/*---------------------------------------------------------------------------*/

void test_new( void );
void test_set( void );
void test_get( void );
void test_chg( void );
void test_init( void );
void test_reinit( void );
void test_sample( void );
void test_validate( void );

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par );

int unur_stringparser_set_verify( UNUR_PAR *par, int );

/*---------------------------------------------------------------------------*/


/* prototypes */
int unur_ssr_set_pedantic( struct unur_par *par, int pedantic );

#define COMPARE_SAMPLE_SIZE   (10000)
#define VIOLATE_SAMPLE_SIZE   (20)





/*---------------------------------------------------------------------------*/

int main()
{ 
	/* open log file for unuran and set output stream for unuran messages */
	UNURANLOG = fopen( "t_stringparser_unuran.log","w" );
	abort_if_NULL( stderr,-1, UNURANLOG );
	unur_set_stream( UNURANLOG );

	/* open log file for testing */
	TESTLOG = fopen( "t_stringparser_test.log","w" );
	abort_if_NULL( stderr,-1, TESTLOG );

	/* write header into log file */
  	{
		time_t started;  
		fprintf(TESTLOG,"\nUNURAN - Universal Non-Uniform RANdom number generator\n\n");
		if (time( &started ) != -1)
			fprintf(TESTLOG,"%s",ctime(&started));
		fprintf(TESTLOG,"\n====================================================\n\n");
	}

        /* seed build-in uniform generators */
        unur_urng_MRG31k3p_seed(739058);
	unur_urng_fish_seed(739058);
	unur_urng_mstd_seed(739058);

	/* seed uniform random number generator */
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
	unur_set_default_urng(prng_new("mt19937(739058)"));
#elif UNUR_URNG_TYPE == UNUR_URNG_RNGSTREAM
	{
	    unsigned long seed[6] = {739058,739058+1,739058+2,739058+3,739058+4,739058+5};
	    RngStream_SetPackageSeed(seed);
	}
#endif  /* UNUR_URNG_TYPE */

	/* set default debugging flag */
	unur_set_default_debug(UNUR_DEBUG_ALL);

	/* start test */
	printf("stringparser: ");

	/* run tests */
	test_new();
	test_set();
	test_get();
	test_chg();
	test_init();
	test_sample();
	test_reinit();
	test_validate();

	/* test finished */
	printf("\n");  fflush(stdout);

	/* close log files */
	fprintf(TESTLOG,"\n====================================================\n\n");
	if (test_ok)
		fprintf(TESTLOG,"All tests PASSED.\n");
	else
		fprintf(TESTLOG,"Test(s) FAILED.\n");

	fclose(UNURANLOG);
	fclose(TESTLOG);

	/* free memory */
        compare_free_memory();

	/* exit */
	exit( (test_ok) ? 0 : -1 );

} /* end of main */

/*---------------------------------------------------------------------------*/

void test_new (void)
{
	/* start test */
	printf("[new "); fflush(stdout);
	fprintf(TESTLOG,"\n[new]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = new  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_new() */

/*---------------------------------------------------------------------------*/

void test_set (void)
{
	/* start test */
	printf("[set "); fflush(stdout);
	fprintf(TESTLOG,"\n[set]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = set  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_set() */

/*---------------------------------------------------------------------------*/

void test_get (void)
{
	/* start test */
	printf("[get "); fflush(stdout);
	fprintf(TESTLOG,"\n[get]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = get  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_get() */

/*---------------------------------------------------------------------------*/

void test_chg (void)
{
	/* start test */
	printf("[chg "); fflush(stdout);
	fprintf(TESTLOG,"\n[chg]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = chg  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_chg() */

/*---------------------------------------------------------------------------*/

void test_init (void)
{
	/* start test */
	printf("[init "); fflush(stdout);
	fprintf(TESTLOG,"\n[init]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* invalid NULL ptr */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 44, (gen = unur_str2gen( NULL )) );
n_tests_failed += check_errorcode( TESTLOG, 44, UNUR_ERR_NULL );
unur_free(gen);
}

/* invalid distribution block */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 51, (gen = unur_str2gen( "xxxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 51, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 55, (gen = unur_str2gen( "method = xxxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 55, UNUR_ERR_STR_SYNTAX );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 59, (gen = unur_str2gen( "normal; xxxx=1" )) );
n_tests_failed += check_errorcode( TESTLOG, 59, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 63, (gen = unur_str2gen( "normal; pv=(1,2,3)" )) );
n_tests_failed += check_errorcode( TESTLOG, 63, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 67, (gen = unur_str2gen( "normal,(1,2)" )) );
n_tests_failed += check_errorcode( TESTLOG, 67, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 71, (gen = unur_str2gen( "beta" )) );
n_tests_failed += check_errorcode( TESTLOG, 71, UNUR_ERR_STR_INVALID );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 75, (gen = unur_str2gen( "beta()" )) );
n_tests_failed += check_errorcode( TESTLOG, 75, UNUR_ERR_STR_INVALID );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 79, (gen = unur_str2gen( "beta(1)" )) );
n_tests_failed += check_errorcode( TESTLOG, 79, UNUR_ERR_STR_INVALID );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "beta(1,2,3)" );
n_tests_failed += check_errorcode( TESTLOG, 83, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
unur_free(gen); gen = NULL;
gen = unur_str2gen( "beta(1,2,3,4,5)" );
n_tests_failed += check_errorcode( TESTLOG, 87, UNUR_ERR_DISTR_NPARAMS );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 91, (gen = unur_str2gen( "gamma(-0.5)" )) );
n_tests_failed += check_errorcode( TESTLOG, 91, UNUR_ERR_STR_INVALID );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 95, (gen = unur_str2gen("normal; orderstatistics=(10,7); domain=(0.,inf)")) );
n_tests_failed += check_errorcode( TESTLOG, 95, UNUR_ERR_STR_SYNTAX );
unur_free(gen);
}

/* invalid other block */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 102, (gen = unur_str2gen( "normal & xxxx = arou" )) );
n_tests_failed += check_errorcode( TESTLOG, 102, UNUR_ERR_STR_UNKNOWN );
unur_free(gen);
}

/* invalid method block */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
gen = unur_str2gen( "normal & method = arou; c = 0" );
n_tests_failed += check_errorcode( TESTLOG, 109, UNUR_ERR_STR_UNKNOWN );

unur_errno = 0;
unur_free(gen); gen = NULL;
n_tests_failed += check_expected_NULL( TESTLOG, 113, (gen = unur_str2gen( "normal & method = xxx" )) );
n_tests_failed += check_errorcode( TESTLOG, 113, UNUR_ERR_NULL );
unur_free(gen);
}

/* invalid urng block */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
gen = unur_str2gen( "normal & urng = xxx" );
n_tests_failed += check_errorcode( TESTLOG, 120, UNUR_ERR_STR );

unur_errno = 0;
unur_free(gen); gen = NULL;
#if UNUR_URNG_TYPE == UNUR_URNG_PRNG
gen = unur_str2gen( "normal & urng = mt19937(123); xxx = 1" );
n_tests_failed += check_errorcode( TESTLOG, 125, UNUR_ERR_STR_UNKNOWN );
#endif
unur_free(gen);
}

/* invalid data */
{
UNUR_GEN   *gen = NULL;
   gen = NULL; 

unur_errno = 0;
n_tests_failed += check_expected_NULL( TESTLOG, 134, (gen = unur_str2gen( "gamma(0.5) & method = tdr" )) );
n_tests_failed += check_errorcode( TESTLOG, 134, UNUR_ERR_GEN_CONDITION );
unur_free(gen);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = init  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_init() */

/*---------------------------------------------------------------------------*/

void test_reinit (void)
{
	/* start test */
	printf("[reinit "); fflush(stdout);
	fprintf(TESTLOG,"\n[reinit]\n");

	/* reset counter */
	n_tests_failed = 0;
  

	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = reinit  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_reinit() */

/*---------------------------------------------------------------------------*/

void test_sample (void)
{
	/* start test */
	printf("[sample "); fflush(stdout);
	fprintf(TESTLOG,"\n[sample]\n");

	/* reset counter */
	n_tests_failed = 0;
  
/* compare function parser - cont */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_normal(NULL,0); 
par = unur_srou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 155, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = srou" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 161, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_ssr_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 168, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = ssr" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 174, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_utdr_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 181, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = utdr" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 187, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_tdr_new(distr);
unur_tdr_set_cpoints(par,8,NULL);
unur_tdr_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 196, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = tdr; cpoints = 8; max_sqhratio = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 202, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_normal(NULL,0); 
par = unur_arou_new(distr);
unur_arou_set_cpoints(par,8,NULL);
unur_arou_set_max_sqhratio(par,0.);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 211, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; pdf = \"(1/sqrt(2*pi)*exp(-x^2/2))\"; mode = 0; pdfarea = 1 & \
	             method = arou; cpoints = 8; max_sqhratio = 0." );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 217, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_cont_new();
unur_distr_cont_set_cdfstr(distr,"1-exp(-x)");
par = unur_ninv_new(distr);
unur_ninv_set_usenewton(par);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 226, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; cdf = \"1-exp(-x)\"; domain = (0,inf)& \
	             method = ninv; usenewton" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 232, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_cont_new();
unur_distr_cont_set_hrstr(distr,"1/(1+x)");
par = unur_hrb_new(distr);
gen = unur_init(par);
n_tests_failed += compare_double_sequence_gen_start( TESTLOG, 240, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "cont; hr = \"1/(1+x)\"; domain = (0,inf)& \
	             method = hrb" );
n_tests_failed += compare_double_sequence_gen( TESTLOG, 246, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}

/* compare function parser - discr */
{
UNUR_DISTR *distr = NULL;
UNUR_PAR   *par = NULL;
UNUR_GEN   *gen = NULL;
   double fpar[] = {0.4};
   distr = NULL;
   par = NULL;
   gen = NULL; 

unur_errno = 0;
distr = unur_distr_geometric(fpar,1); 
par = unur_dsrou_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 264, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dsrou" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 270, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dau_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 277, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dau" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 283, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dgt_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 290, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dgt" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 296, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
distr = unur_distr_geometric(fpar,1); 
par = unur_dari_new(distr);
gen = unur_init(par);
n_tests_failed += compare_int_sequence_gen_start( TESTLOG, 303, gen, COMPARE_SAMPLE_SIZE );

unur_errno = 0;
unur_free(gen); gen = NULL;
unur_distr_free(distr); distr = NULL;
gen = unur_str2gen( "discr; pmf = \"0.4 * (1-0.4)^k\"; domain = (0,inf); mode = 0; pmfsum = 1 & \
	             method = dari" );
n_tests_failed += compare_int_sequence_gen( TESTLOG, 309, gen, COMPARE_SAMPLE_SIZE );
unur_free(gen);
unur_distr_free(distr);
}


	/* test finished */
	test_ok &= (n_tests_failed) ? 0 : 1;
	(n_tests_failed) ? printf("--> failed] ") : printf("--> ok] ");

#if WITH_DMALLOC
	dmalloc_vmessage("section = sample  ================================\n",NULL);
	dmalloc_log_unfreed();
#endif

} /* end of test_sample() */

/*---------------------------------------------------------------------------*/

void test_validate (void)
{
	UNUR_DISTR *distr[1];
	UNUR_PAR *par;
	UNUR_GEN *gen;
	int n_tests_failed = 0;
	double *darray;
	double fpm[10];

	/* start test */
	printf("[validate "); fflush(stdout);
	fprintf(TESTLOG,"\n[validate]\n");

	/* reset counter */
	n_tests_failed = 0;


/* distributions: 0 */

/* free distributions */
#if WITH_DMALLOC
	dmalloc_vmessage("section = validate  ================================\n","");
	dmalloc_log_unfreed();
#endif


	/* test finished */
	test_ok &= (n_tests_failed>2) ? 0 : 1;   /* we accept 2 failure */
	(n_tests_failed>2) ? printf(" --> failed] ") : printf(" --> ok] ");

	/* prevent compiler from making useless annoying warnings */
	distr[0] = NULL;
	par = NULL;
	gen = NULL;
	darray = NULL;
	fpm[0] = 0.;

} /* end of test_validate */

/*---------------------------------------------------------------------------*/





/*---------------------------------------------------------------------------*/
/* run generator in verifying mode */

void run_verify_generator( FILE *LOG, int line, UNUR_PAR *par )
{
	UNUR_GEN *gen;
	int i;

	/* switch to verifying mode */
	unur_stringparser_set_verify(par,1);

	/* initialize generator */
	gen = unur_init( par ); abort_if_NULL(LOG, line, gen);

	/* run generator */
	for (i=0; i<VIOLATE_SAMPLE_SIZE; i++)
		unur_sample_cont(gen);

	/* destroy generator */
	unur_free(gen); 

} /* end of run_verify_generator() */

int unur_stringparser_set_verify(UNUR_PAR *par, int verify) {return 0;}
